#pragma once

#include <map>
#include <vector>
#include "XMLExceptions.h"

#ifdef _93E6005A_7D4E_40cc_958A_5873BF97537F
#define EXPCLASS __declspec(dllexport)
#else
#define EXPCLASS
#endif

class XMLTag;


/****************************************************************************
*                                 class Value
****************************************************************************/

class EXPCLASS Value {
    protected:
       CString value;
       int line;
    public:
       BOOL IsEmpty() const { return value.IsEmpty(); }
       Value(const CString & val, int ln) { value = val; line = ln; }
       CString GetValue() const { return value; }
       void SetValue(const CString & val) { value = val; }
       int GetLine() const { return line; }
};

/****************************************************************************
*                                   XMLNode
* <node {key = value}* >
*    {children}*
* </node>
* OR
* <node {key = value}* />
****************************************************************************/

class EXPCLASS XMLNode {
    public:
       XMLNode() { Init(0); }
       XMLNode(int ln) { Init(ln); }
       virtual ~XMLNode() {
          for(std::vector<XMLNode*>::iterator iter = Children.begin(); iter != Children.end(); ++iter)
             { /* delete each child */
              delete *iter;
             } /* delete each child */

          for(XMLValue::iterator iter = Values.begin(); iter != Values.end(); ++iter)
             { /* delete each value */
              delete iter->second;
             } /* delete each value */
       }
       virtual void UnParse(CString & result, BOOL CRLF, int depth) const;
       int GetLine() const { return line; }
    public:
       static XMLNode * Copy(const XMLNode * node);
       static XMLTag * Copy(const XMLTag * node) { return (XMLTag *)XMLNode::Copy((const XMLNode *)node); }
       static CString cvStringToXML(const CString & text);
       BOOL Replace(XMLNode * OldNode, XMLNode * NewNode);
       void AddChild(XMLNode * child);
       void AddValue(const CString & nm, const CString & val, int ln = 0);
       BOOL IsValue(const CString & nm) const;
       CString GetValue(const CString & nm) const;
       void RemoveValue(const CString & nm) {
                                             Values.erase(nm);
                                            }
       
       BOOL HasValues() const { return !Values.empty(); } 

       
       BOOL SetValue(const CString & nm, const CString & val, int ln = 0);
       static BOOL IsTrue(const CString & val);

       BOOL Remove(XMLNode * child);
       void RemoveAllChildren();
    public:
       typedef std::vector<XMLNode *> ChildNodes;
       ChildNodes Children;
       LPVOID UserData;
       typedef std::map<CString, Value *, std::less<CString> > XMLValue;
       XMLValue Values;
       void Init(int ln) { line = ln; UserData = NULL; }

       void CopyValues(const XMLNode * node);
       void CopyChildren(const XMLNode * node);
    protected:
       void UnParseValues(CString & result, BOOL CRLF, int depth) const;
       void UnParseChildren(CString & result, BOOL CRLF, int depth) const;
       int line;
};       

/****************************************************************************
*                                 XMLComment
* <-- comment text -->
****************************************************************************/

class EXPCLASS XMLComment : public XMLNode {
    public:
       XMLComment(const CString & tx, int line = 0) : XMLNode(line) { text = tx; }
       virtual void UnParse(CString & result, BOOL CRLF, int depth) const;
       CString GetComment() const { return text; }
    protected:
       CString text;
};

/****************************************************************************
*                                  clas XMLHeader
****************************************************************************/

class EXPCLASS XMLHeader : public XMLNode {
    public:
       XMLHeader(int line) : XMLNode(line) {}
       virtual void UnParse(CString & result, BOOL CRLF, int depth) const;
}; // XMLHeader

/****************************************************************************
*                                   XMLRoot
* The root node
****************************************************************************/

class EXPCLASS XMLRoot : public XMLNode  {
    public:
       XMLRoot() {}
       virtual void UnParse(CString & result, BOOL CRLF, int depth) const;
}; // XMLRoot

/****************************************************************************
*                                   XMLTag
* <tag::value>
*
****************************************************************************/

class EXPCLASS XMLTag : public XMLNode {
    public:
       XMLTag() : XMLNode() {}
       XMLTag(int ln) : XMLNode(ln) {}
       XMLTag(const CString & tg, const CString & nm, int ln = 0)
          : XMLNode(ln) {
          Tag = tg;
          Name = nm;
       };
       CString GetTag() const { return Tag; }
       CString GetName() const { return Name; }
       virtual void UnParse(CString & result, BOOL CRLF, int depth) const;
    protected:
       CString Tag;
       CString Name;
};
